@@ -15,17 +15,6 @@ typedef long		int32;		/* -100,000 .. 100,000 */
 typedef unsigned long	uint32;		/* 	  0 ..  2^31-1 */
 
 
-/*
- * 	Define NO_VARARGS if the varargs feature is not available
- *
- *	Also define NO_VARARGS if the vprintf/vsprintf routines are not
- *	available (however, this will only by safe on some machines, like
- *	the VAX).
- *
- */
-
-#define NO_VARARGS
-
 /*
  *	Define STRCSPN if the strcspn() function is not available.
  */
