@@ -43,13 +43,11 @@ typedef struct direct Direntry;		/* BSD */
  *	NNTP support requires tcp/ip with socket interface.
  *
  *	Define NO_RENAME if the rename() system call is not available.
- *	Define NO_BZERO if the bzero() function is not available.
  *	Define EXCELAN if the tcp/ip package is EXCELAN based.
  *	Define NNTP_EXTRA_LIB to any libraries required only for nntp.
  */
 
 /* #define NO_RENAME			/* */
-/* #define NO_BZERO			/* */
 /* #define EXCELAN			/* */
 /* #define NNTP_EXTRA_LIB -lsocket	/* */
 
