@@ -1,20 +1,23 @@
 /*
  *	(c) Copyright 1990, Kim Fabricius Storm.  All rights reserved.
+ *      Copyright (c) 1996-2005 Michael T Pins.  All rights reserved.
  *
  *	Pack subject by eliminating RE prefixes and - (nf) suffixes.
  *	Also collapse multiple blanks into single blanks.
  */
 
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
 #include "config.h"
+#include "global.h"
 
 int
-pack_subject(dest, src, re_counter_ptr, max_length)
-register char *dest, *src;
-int *re_counter_ptr, max_length;
+pack_subject(register char *dest, register char *src, int *re_counter_ptr, int max_length)
 {
-    register char *max_dest;
-    int re = 0;
-    char *start_dest = dest;
+    register char  *max_dest;
+    int             re = 0;
+    char           *start_dest = dest;
 
     if (src) {
 	max_dest = dest + max_length;
@@ -24,13 +27,14 @@ int *re_counter_ptr, max_length;
 		src++;
 		continue;
 	    }
-
 	    /* count and remove 'Re: Re: ...' */
 
-	    if (*src != 'R' && *src != 'r') break;
+	    if (*src != 'R' && *src != 'r')
+		break;
 	    *dest++ = *src++;
 
-	    if (*src != 'e' && *src != 'E') break;
+	    if (*src != 'e' && *src != 'E')
+		break;
 	    *dest++ = *src++;
 
 	    if (*src == ':' || *src == ' ') {
@@ -39,13 +43,14 @@ int *re_counter_ptr, max_length;
 		re++;
 		continue;
 	    }
-
-	    if (*src != '^') break;
+	    if (*src != '^')
+		break;
 
 	    src++;
 	    dest = start_dest;
 
-	    while (isdigit(*src)) *dest++ = *src++;
+	    while (isdigit(*src))
+		*dest++ = *src++;
 	    if (dest == start_dest)
 		re++;
 	    else {
@@ -53,22 +58,24 @@ int *re_counter_ptr, max_length;
 		dest = start_dest;
 		re += atoi(dest);
 	    }
-	    if (*src == ':') src++;
+	    if (*src == ':')
+		src++;
 	}
 
 	while (*src && dest < max_dest) {
-	    if (*src == '-' && strncmp("- (nf)", src, 5) == 0) break;
+	    if (*src == '-' && strncmp("- (nf)", src, 5) == 0)
+		break;
 	    if (isascii(*src) && isspace(*src)) {
-		do src++;
+		do
+		    src++;
 		while (isascii(*src) && isspace(*src));
 		*dest++ = SP;
 	    } else
 		*dest++ = *src++;
 	}
     }
-
     *dest = NUL;
-    *re_counter_ptr = (char)re;
+    *re_counter_ptr = (char) re;
 
     return dest - start_dest;
 }
