@@ -1,19 +1,5 @@
 /*
- * regexp.c - regular expression matching
- *
- * NOTICE: THIS CODE HAS BEEN MODIFIED TO FIT THE NN ENVIRONMENT.
- *
- * DESCRIPTION
- *
- *	This source was taken from the pax posting in comp.sources.unix.
- *
- *	Underneath the reformatting and comment blocks which were added to
- *	make it consistent with the rest of the code, you will find a
- *	modified version of Henry Specer's regular expression library.
- *	Henry's functions were modified to provide the minimal regular
- *	expression matching, as required by P1003.  Henry's code was
- *	copyrighted, and copy of the copyright message and restrictions
- *	are provided, verbatim, below:
+ * @(#)regexp.c	1.3 of 18 April 87
  *
  *	Copyright (c) 1986 by University of Toronto.
  *	Written by Henry Spencer.  Not derived from licensed software.
@@ -23,75 +9,35 @@
  *	subject to the following restrictions:
  *
  *	1. The author is not responsible for the consequences of use of
- *         this software, no matter how awful, even if they arise
- *	   from defects in it.
+ *		this software, no matter how awful, even if they arise
+ *		from defects in it.
  *
  *	2. The origin of this software must not be misrepresented, either
- *	   by explicit claim or by omission.
+ *		by explicit claim or by omission.
  *
  *	3. Altered versions must be plainly marked as such, and must not
- *	   be misrepresented as being the original software.
- *
- * 	Beware that some of this code is subtly aware of the way operator
- * 	precedence is structured in regular expressions.  Serious changes in
- * 	regular-expression syntax might require a total rethink.
- *
- * AUTHORS
- *
- *     Mark H. Colburn, NAPS International (mark@jhereg.mn.org)
- *     Henry Spencer, University of Torronto (henry@utzoo.edu)
- *
- * Sponsored by The USENIX Association for public distribution.
- *
- * $Log:	regexp.c,v $
- * Revision 1.1  88/12/23  18:02:32  mark
- * Initial revision
+ *		be misrepresented as being the original software.
  *
+ * Beware that some of this code is subtly aware of the way operator
+ * precedence is structured in regular expressions.  Serious changes in
+ * regular-expression syntax might require a total rethink.
  */
 
-#define NN
+/*
+ * NOTICE: THIS CODE HAS BEEN MODIFIED TO FIT THE NN ENVIRONMENT.
+ *
+ * modified by Kim Fabricius Storm
+ * modified by Michael T. Pins
+ */
 
-/* Headers */
-
-#ifdef NN
+#include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
 #include "config.h"
+#include "global.h"
 #include "regexp.h"
 #include "nn_term.h"
-#include <ctype.h>
-
-/* regexp.c */
-
-static char *reg __APROTO((int paren, int *flagp));
-static char *regbranch __APROTO((int *flagp));
-static char *regpiece __APROTO((int *flagp));
-static char *regatom __APROTO((int *flagp));
-static char *regnode __APROTO((int op));
-static void regc __APROTO((int b));
-static void reginsert __APROTO((int op, char *opnd));
-static void regtail __APROTO((char *p, char *val));
-static void regoptail __APROTO((char *p, char *val));
-static int regtry __APROTO((regexp *prog, char *string));
-static int regmatch __APROTO((char *prog));
-static int regrepeat __APROTO((char *p));
-static char *regnext __APROTO((register char *p));
-
-#else
-#include "pax.h"
-
-#ifndef lint
-static char    *Ident = "$Id: regexp.c,v 1.1 88/12/23 18:02:32 mark Rel $";
-#endif
-#endif
-
-#ifndef HAVE_STDLIB_H
-extern void    *malloc();
-#endif
-#ifndef HAVE_STRING_H
-extern char    *strchr();
-extern char    *strncpy();
-#endif
-
-
 
 /*
  * The "internal use only" fields in regexp.h are present to pass info from
@@ -117,8 +63,8 @@ extern char    *strncpy();
  * Structure for regexp "program".  This is essentially a linear encoding
  * of a nondeterministic finite-state machine (aka syntax charts or
  * "railroad normal form" in parsing technology).  Each node is an opcode
- * plus a "nxt" pointer, possibly plus an operand.  "Nxt" pointers of
- * all nodes except BRANCH implement concatenation; a "nxt" pointer with
+ * plus a "next" pointer, possibly plus an operand.  "Next" pointers of
+ * all nodes except BRANCH implement concatenation; a "next" pointer with
  * a BRANCH on both ends of it is connecting two alternatives.  (Here we
  * have one of the subtle syntax dependencies:  an individual BRANCH (as
  * opposed to a collection of them) is never concatenated with anything
@@ -138,16 +84,16 @@ extern char    *strncpy();
 #define	ANYBUT	5		/* str	Match any character not in this
 				 * string. */
 #define	BRANCH	6		/* node	Match this alternative, or the
-				 * nxt... */
-#define	BACK	7		/* no	Match "", "nxt" ptr points backward. */
+				 * next... */
+#define	BACK	7		/* no	Match "", "next" ptr points backward. */
 #define	EXACTLY	8		/* str	Match this string. */
 #define	NOTHING	9		/* no	Match empty string. */
 #define	STAR	10		/* node	Match this (simple) thing 0 or more
 				 * times. */
-#ifdef NN
-#define W_START	11		/* no	Match beginning of word */
-#define W_END	12		/* no	Match end of word */
-#endif
+#define	PLUS	11		/* node	Match this (simple) thing 1 or more
+				 * times. */
+#define W_START 12		/* no	Match beginning of word */
+#define W_END	13		/* no	Match end of word */
 #define	OPEN	20		/* no	Mark this point in input as start of
 				 * #n. */
  /* OPEN+1 is number 1, etc. */
@@ -157,57 +103,59 @@ extern char    *strncpy();
  * Opcode notes:
  *
  * BRANCH	The set of branches constituting a single choice are hooked
- *		together with their "nxt" pointers, since precedence prevents
+ *		together with their "next" pointers, since precedence prevents
  *		anything being concatenated to any individual branch.  The
- *		"nxt" pointer of the last BRANCH in a choice points to the
+ *		"next" pointer of the last BRANCH in a choice points to the
  *		thing following the whole choice.  This is also where the
- *		final "nxt" pointer of each individual branch points; each
+ *		final "next" pointer of each individual branch points; each
  *		branch starts with the operand node of a BRANCH node.
  *
- * BACK		Normal "nxt" pointers all implicitly point forward; BACK
+ * BACK		Normal "next" pointers all implicitly point forward; BACK
  *		exists to make loop structures possible.
  *
- * STAR		complex '*', are implemented as circular BRANCH structures
- *		using BACK.  Simple cases (one character per match) are
- *		implemented with STAR for speed and to minimize recursive
- *		plunges.
+ * STAR,PLUS	'?', and complex '*' and '+', are implemented as circular
+ *		BRANCH structures using BACK.  Simple cases (one character
+ *		per match) are implemented with STAR and PLUS for speed
+ *		and to minimize recursive plunges.
  *
  * OPEN,CLOSE	...are numbered at compile time.
  */
 
 /*
- * A node is one char of opcode followed by two chars of "nxt" pointer.
- * "Nxt" pointers are stored as two 8-bit pieces, high order first.  The
+ * A node is one char of opcode followed by two chars of "next" pointer.
+ * "Next" pointers are stored as two 8-bit pieces, high order first.  The
  * value is a positive offset from the opcode of the node containing it.
  * An operand, if any, simply follows the node.  (Note that much of the
  * code generation knows about this implicit relationship.)
  *
- * Using two bytes for the "nxt" pointer is vast overkill for most things,
+ * Using two bytes for the "next" pointer is vast overkill for most things,
  * but allows patterns to get big without disasters.
  */
 #define	OP(p)	(*(p))
 #define	NEXT(p)	(((*((p)+1)&0377)<<8) + (*((p)+2)&0377))
 #define	OPERAND(p)	((p) + 3)
 
+
 /*
  * Utility definitions.
  */
 
-#define	FAIL(m)	{ regerror(m); return(NULL); }
-#define	ISMULT(c)	((c) == '*')
-#define	META	"^$.[()|*\\"
 #ifndef CHARBITS
 #define	UCHARAT(p)	((int)*(unsigned char *)(p))
 #else
 #define	UCHARAT(p)	((int)*(p)&CHARBITS)
 #endif
 
+#define	FAIL(m)	{ regerror(m); return(NULL); }
+#define	ISMULT(c)	((c) == '*' || (c) == '+' || (c) == '?')
+#define	META	"^$.[()|?+*\\"
+
 /*
  * Flags to be passed up and down.
  */
 #define	HASWIDTH	01	/* Known never to match null string. */
-#define	SIMPLE		02	/* Simple enough to be STAR operand. */
-#define	SPSTART		04	/* Starts with * */
+#define	SIMPLE		02	/* Simple enough to be STAR/PLUS operand. */
+#define	SPSTART		04	/* Starts with * or +. */
 #define	WORST		0	/* Worst case. */
 
 /*
@@ -222,21 +170,25 @@ static long     regsize;	/* Code size. */
 /*
  * Forward declarations for regcomp()'s friends.
  */
+
 #ifndef STATIC
 #define	STATIC	static
 #endif
-STATIC char    *reg();
-STATIC char    *regbranch();
-STATIC char    *regpiece();
-STATIC char    *regatom();
-STATIC char    *regnode();
-STATIC char    *regnext();
-STATIC void     regc();
-STATIC void     reginsert();
-STATIC void     regtail();
-STATIC void     regoptail();
+
+STATIC char    *reg(int, int *);
+STATIC char    *regbranch(int *);
+STATIC char    *regpiece(int *);
+STATIC char    *regatom(int *);
+STATIC char    *regnode(char);
+STATIC char    *regnext(register char *);
+STATIC void     regc(char);
+STATIC void     reginsert(char, char *);
+STATIC void     regtail(char *, char *);
+STATIC void     regoptail(char *, char *);
+STATIC void     regerror(char *);
+
 #ifdef STRCSPN
-STATIC int      strcspn();
+STATIC int      strcspn(char *, char *);
 #endif
 
 /*
@@ -254,8 +206,8 @@ STATIC int      strcspn();
  * Beware that the optimization-preparation code in here knows about some
  * of the structure of the compiled regexp.
  */
-regexp *regcomp(exp)
-char           *exp;
+regexp         *
+regcomp(char *exp)
 {
     register regexp *r;
     register char  *scan;
@@ -271,7 +223,7 @@ char           *exp;
     regnpar = 1;
     regsize = 0L;
     regcode = &regdummy;
-    regc(MAGIC);
+    regc((char) MAGIC);
     if (reg(0, &flags) == NULL)
 	return (NULL);
 
@@ -288,7 +240,7 @@ char           *exp;
     regparse = exp;
     regnpar = 1;
     regcode = r->program;
-    regc(MAGIC);
+    regc((char) MAGIC);
     if (reg(0, &flags) == NULL)
 	return (NULL);
 
@@ -319,7 +271,7 @@ char           *exp;
 	    longest = NULL;
 	    len = 0;
 	    for (; scan != NULL; scan = regnext(scan))
-		if (OP(scan) == EXACTLY && (int)strlen(OPERAND(scan)) >= len) {
+		if (OP(scan) == EXACTLY && strlen(OPERAND(scan)) >= len) {
 		    longest = OPERAND(scan);
 		    len = strlen(OPERAND(scan));
 		}
@@ -338,10 +290,11 @@ char           *exp;
  * Combining parenthesis handling with the base level of regular expression
  * is a trifle forced, but the need to tie the tails of the branches to what
  * follows makes it hard to avoid.
+ *
+ * paren: Parenthesized?
  */
-static char *reg(paren, flagp)
-int             paren;		/* Parenthesized? */
-int            *flagp;
+static char    *
+reg(int paren, int *flagp)
 {
     register char  *ret;
     register char  *br;
@@ -409,8 +362,8 @@ int            *flagp;
  *
  * Implements the concatenation operator.
  */
-static char  *regbranch(flagp)
-int            *flagp;
+static char    *
+regbranch(int *flagp)
 {
     register char  *ret;
     register char  *chain;
@@ -433,24 +386,26 @@ int            *flagp;
 	chain = latest;
     }
     if (chain == NULL)		/* Loop ran zero times. */
-	regnode(NOTHING);
+	(void) regnode(NOTHING);
 
     return (ret);
 }
 
 /*
- - regpiece - something followed by possible [*]
+ - regpiece - something followed by possible [*+?]
  *
- * Note that the branching code sequence used for * is somewhat optimized:
- * they use the same NOTHING node as both the endmarker for their branch
- * list and the body of the last branch.  It might seem that this node could
- * be dispensed with entirely, but the endmarker role is not redundant.
+ * Note that the branching code sequences used for ? and the general cases
+ * of * and + are somewhat optimized:  they use the same NOTHING node as
+ * both the endmarker for their branch list and the body of the last branch.
+ * It might seem that this node could be dispensed with entirely, but the
+ * endmarker role is not redundant.
  */
-static char *regpiece(flagp)
-int            *flagp;
+static char    *
+regpiece(int *flagp)
 {
     register char  *ret;
     register char   op;
+    register char  *next;
     int             flags;
 
     ret = regatom(&flags);
@@ -462,9 +417,9 @@ int            *flagp;
 	*flagp = flags;
 	return (ret);
     }
-    if (!(flags & HASWIDTH))
-	FAIL("* operand could be empty");
-    *flagp = (WORST | SPSTART);
+    if (!(flags & HASWIDTH) && op != '?')
+	FAIL("*+ operand could be empty");
+    *flagp = (op != '+') ? (WORST | SPSTART) : (WORST | HASWIDTH);
 
     if (op == '*' && (flags & SIMPLE))
 	reginsert(STAR, ret);
@@ -475,10 +430,26 @@ int            *flagp;
 	regoptail(ret, ret);	/* back */
 	regtail(ret, regnode(BRANCH));	/* or */
 	regtail(ret, regnode(NOTHING));	/* null. */
+    } else if (op == '+' && (flags & SIMPLE))
+	reginsert(PLUS, ret);
+    else if (op == '+') {
+	/* Emit x+ as x(&|), where & means "self". */
+	next = regnode(BRANCH);	/* Either */
+	regtail(ret, next);
+	regtail(regnode(BACK), ret);	/* loop back */
+	regtail(next, regnode(BRANCH));	/* or */
+	regtail(ret, regnode(NOTHING));	/* null. */
+    } else if (op == '?') {
+	/* Emit x? as (x|) */
+	reginsert(BRANCH, ret);	/* Either x */
+	regtail(ret, regnode(BRANCH));	/* or */
+	next = regnode(NOTHING);/* null. */
+	regtail(ret, next);
+	regoptail(ret, next);
     }
     regparse++;
     if (ISMULT(*regparse))
-	FAIL("nested *");
+	FAIL("nested *?+");
 
     return (ret);
 }
@@ -491,8 +462,8 @@ int            *flagp;
  * faster to run.  Backslashed characters are exceptions, each becoming a
  * separate node; the code is simpler that way and it's not worth fixing.
  */
-static char *regatom(flagp)
-int            *flagp;
+static char    *
+regatom(int *flagp)
 {
     register char  *ret;
     int             flags;
@@ -500,115 +471,108 @@ int            *flagp;
     *flagp = WORST;		/* Tentatively. */
 
     switch (*regparse++) {
-    case '^':
-	ret = regnode(BOL);
-	break;
-    case '$':
-	ret = regnode(EOL);
-	break;
-    case '.':
-	ret = regnode(ANY);
-	*flagp |= HASWIDTH | SIMPLE;
-	break;
-    case '[':{
-	    register int    class;
-	    register int    classend;
+	case '^':
+	    ret = regnode(BOL);
+	    break;
+	case '$':
+	    ret = regnode(EOL);
+	    break;
+	case '.':
+	    ret = regnode(ANY);
+	    *flagp |= HASWIDTH | SIMPLE;
+	    break;
+	case '[':{
+		register int    class;
+		register int    classend;
 
-	    if (*regparse == '^') {	/* Complement of range. */
-		ret = regnode(ANYBUT);
-		regparse++;
-	    } else
-		ret = regnode(ANYOF);
-	    if (*regparse == ']' || *regparse == '-')
-		regc(*regparse++);
-	    while (*regparse != '\0' && *regparse != ']') {
-		if (*regparse == '-') {
+		if (*regparse == '^') {	/* Complement of range. */
+		    ret = regnode(ANYBUT);
 		    regparse++;
-		    if (*regparse == ']' || *regparse == '\0')
-			regc('-');
-		    else {
-			class = UCHARAT(regparse - 2) + 1;
-			classend = UCHARAT(regparse);
-			if (class > classend + 1)
-			    FAIL("invalid [] range");
-			for (; class <= classend; class++)
-			    regc(class);
-			regparse++;
-		    }
 		} else
+		    ret = regnode(ANYOF);
+		if (*regparse == ']' || *regparse == '-')
 		    regc(*regparse++);
+		while (*regparse != '\0' && *regparse != ']') {
+		    if (*regparse == '-') {
+			regparse++;
+			if (*regparse == ']' || *regparse == '\0')
+			    regc('-');
+			else {
+			    class = UCHARAT(regparse - 2) + 1;
+			    classend = UCHARAT(regparse);
+			    if (class > classend + 1)
+				FAIL("invalid [] range");
+			    for (; class <= classend; class++)
+				regc(class);
+			    regparse++;
+			}
+		    } else
+			regc(*regparse++);
+		}
+		regc('\0');
+		if (*regparse != ']')
+		    FAIL("unmatched []");
+		regparse++;
+		*flagp |= HASWIDTH | SIMPLE;
 	    }
-	    regc('\0');
-	    if (*regparse != ']')
-		FAIL("unmatched []");
-	    regparse++;
-	    *flagp |= HASWIDTH | SIMPLE;
-	}
-	break;
-    case '(':
-	ret = reg(1, &flags);
-	if (ret == NULL)
-	    return (NULL);
-	*flagp |= flags & (HASWIDTH | SPSTART);
-	break;
-    case '\0':
-    case '|':
-    case ')':
-	FAIL("internal urp");	/* Supposed to be caught earlier. */
-	break;
-    case '*':
-	FAIL("* follows nothing");
-	break;
-    case '\\':
-#ifdef NN
-	switch (*regparse++) {
-	 case '\0':
-	    FAIL("trailing \\");
-	    break;	/* for lint, we can't actually get here */
-	 case '<':
-	    ret = regnode(W_START);
 	    break;
-	 case '>':
-	    ret = regnode(W_END);
+	case '(':
+	    ret = reg(1, &flags);
+	    if (ret == NULL)
+		return (NULL);
+	    *flagp |= flags & (HASWIDTH | SPSTART);
 	    break;
-	 default:
-	    ret = regnode(EXACTLY);
-	    regc(*(regparse - 1));
-	    regc('\0');
-	    *flagp |= HASWIDTH | SIMPLE;
+	case '\0':
+	case '|':
+	case ')':
+	    FAIL("internal urp");	/* Supposed to be caught earlier. */
 	    break;
-	}
-#else
-	if (*regparse == '\0')
-	    FAIL("trailing \\");
-	ret = regnode(EXACTLY);
-	regc(*regparse++);
-	regc('\0');
-	*flagp |= HASWIDTH | SIMPLE;
-#endif	
-	break;
-    default:{
-	    register int    len;
-	    register char   ender;
+	case '?':
+	case '+':
+	case '*':
+	    FAIL("?+* follows nothing");
+	    break;
+	case '\\':
+	    switch (*regparse++) {
+		case '\0':
+		    FAIL("trailing \\");
+		    break;	/* for lint, we can't actually get here */
+		case '<':
+		    ret = regnode(W_START);
+		    break;
+		case '>':
+		    ret = regnode(W_END);
+		    break;
+		default:
+		    ret = regnode(EXACTLY);
+		    regc(*regparse);
+		    regc('\0');
+		    *flagp |= HASWIDTH | SIMPLE;
+		    break;
+	    }
+	    break;
+	default:{
+		register int    len;
+		register char   ender;
 
-	    regparse--;
-	    len = strcspn(regparse, META);
-	    if (len <= 0)
-		FAIL("internal disaster");
-	    ender = *(regparse + len);
-	    if (len > 1 && ISMULT(ender))
-		len--;		/* Back off clear of * operand. */
-	    *flagp |= HASWIDTH;
-	    if (len == 1)
-		*flagp |= SIMPLE;
-	    ret = regnode(EXACTLY);
-	    while (len > 0) {
-		regc(*regparse++);
-		len--;
+		regparse--;
+		len = strcspn(regparse, META);
+		if (len <= 0)
+		    FAIL("internal disaster");
+		ender = *(regparse + len);
+		if (len > 1 && ISMULT(ender))
+		    len--;	/* Back off clear of ?+* operand. */
+		*flagp |= HASWIDTH;
+		if (len == 1)
+		    *flagp |= SIMPLE;
+		ret = regnode(EXACTLY);
+		while (len > 0) {
+		    regc(*regparse++);
+		    len--;
+		}
+		regc('\0');
 	    }
-	    regc('\0');
-	}
-	break;
+	    break;
     }
 
     return (ret);
@@ -617,8 +581,8 @@ int            *flagp;
 /*
  - regnode - emit a node
  */
-static char *regnode(op)
-char            op;
+static char    *		/* Location. */
+regnode(char op)
 {
     register char  *ret;
     register char  *ptr;
@@ -630,7 +594,7 @@ char            op;
     }
     ptr = ret;
     *ptr++ = op;
-    *ptr++ = '\0';		/* Null "nxt" pointer. */
+    *ptr++ = '\0';		/* Null "next" pointer. */
     *ptr++ = '\0';
     regcode = ptr;
 
@@ -640,8 +604,8 @@ char            op;
 /*
  - regc - emit (if appropriate) a byte of code
  */
-static void regc(b)
-char            b;
+static void
+regc(char b)
 {
     if (regcode != &regdummy)
 	*regcode++ = b;
@@ -654,9 +618,8 @@ char            b;
  *
  * Means relocating the operand.
  */
-static void reginsert(op, opnd)
-char            op;
-char           *opnd;
+static void
+reginsert(char op, char *opnd)
 {
     register char  *src;
     register char  *dst;
@@ -681,9 +644,8 @@ char           *opnd;
 /*
  - regtail - set the next-pointer at the end of a node chain
  */
-static void regtail(p, val)
-char           *p;
-char           *val;
+static void
+regtail(char *p, char *val)
 {
     register char  *scan;
     register char  *temp;
@@ -712,9 +674,8 @@ char           *val;
 /*
  - regoptail - regtail on operand of first argument; nop if operandless
  */
-static void regoptail(p, val)
-char           *p;
-char           *val;
+static void
+regoptail(char *p, char *val)
 {
     /* "Operandless" and "op != BRANCH" are synonymous in practice. */
     if (p == NULL || p == &regdummy || OP(p) != BRANCH)
@@ -737,22 +698,21 @@ static char   **regendp;	/* Ditto for endp. */
 /*
  * Forwards.
  */
-STATIC int      regtry();
-STATIC int      regmatch();
-STATIC int      regrepeat();
+STATIC int      regtry(regexp *, char *);
+STATIC int      regmatch(char *);
+STATIC int      regrepeat(char *);
 
 #ifdef DEBUG
 int             regnarrate = 0;
-void            regdump();
-STATIC char    *regprop();
+void            regdump(regexp *);
+STATIC char    *regprop(char *);
 #endif
 
 /*
  - regexec - match a regexp against a string
  */
-int regexec(prog, string)
-register regexp *prog;
-register char  *string;
+int
+regexec(register regexp * prog, register char *string)
 {
     register char  *s;
 
@@ -807,17 +767,8 @@ register char  *string;
 /*
  - regtry - try match at specific point
  */
-#ifdef __STDC__
-
-static int regtry(regexp *prog, char *string)
-
-#else
-
-static int regtry(prog, string)
-regexp         *prog;
-char           *string;
-
-#endif
+static int			/* 0 failure, 1 success */
+regtry(regexp * prog, char *string)
 {
     register int    i;
     register char **sp;
@@ -851,193 +802,195 @@ char           *string;
  * need to know whether the rest of the match failed) by a loop instead of
  * by recursion.
  */
-#ifdef __STDC__
-
-static int regmatch(char *prog)
-
-#else
-
-static int regmatch(prog)
-char           *prog;
-
-#endif
+static int			/* 0 failure, 1 success */
+regmatch(char *prog)
 {
     register char  *scan;	/* Current node. */
-    char           *nxt;	/* nxt node. */
+    char           *next;	/* Next node. */
 
     scan = prog;
+
 #ifdef DEBUG
     if (scan != NULL && regnarrate)
 	fprintf(stderr, "%s(\n", regprop(scan));
 #endif
+
     while (scan != NULL) {
+
 #ifdef DEBUG
 	if (regnarrate)
 	    fprintf(stderr, "%s...\n", regprop(scan));
 #endif
-	nxt = regnext(scan);
+
+	next = regnext(scan);
 
 	switch (OP(scan)) {
-	case BOL:
-	    if (reginput != regbol)
-		return (0);
-	    break;
-	case EOL:
-	    if (*reginput != '\0')
-		return (0);
-	    break;
-#ifdef NN
-	case W_START:
-	    if (*reginput == '\0' || isspace(*reginput))
-		return (0);
-	    if (reginput != regbol && !isspace(*(reginput - 1))) 
-		return (0);
-	    break;
-	case W_END:
-	    if (*reginput != '\0' && !isspace(*reginput))
-		return (0);
-	    if (reginput == regbol || isspace(*(reginput - 1)))
-		return (0);
-	    break;
-#endif
-	case ANY:
-	    if (*reginput == '\0')
-		return (0);
-	    reginput++;
-	    break;
-	case EXACTLY:{
-		register int    len;
-		register char  *opnd;
-
-		opnd = OPERAND(scan);
-		/* Inline the first character, for speed. */
-		if (*opnd != *reginput)
+	    case BOL:
+		if (reginput != regbol)
 		    return (0);
-		len = strlen(opnd);
-		if (len > 1 && strncmp(opnd, reginput, len) != 0)
+		break;
+	    case EOL:
+		if (*reginput != '\0')
 		    return (0);
-		reginput += len;
-	    }
-	    break;
-	case ANYOF:
-	    if (*reginput == '\0' || strchr(OPERAND(scan), *reginput) == NULL)
-		return (0);
-	    reginput++;
-	    break;
-	case ANYBUT:
-	    if (*reginput == '\0' || strchr(OPERAND(scan), *reginput) != NULL)
-		return (0);
-	    reginput++;
-	    break;
-	case NOTHING:
-	    break;
-	case BACK:
-	    break;
-	case OPEN + 1:
-	case OPEN + 2:
-	case OPEN + 3:
-	case OPEN + 4:
-	case OPEN + 5:
-	case OPEN + 6:
-	case OPEN + 7:
-	case OPEN + 8:
-	case OPEN + 9:{
-		register int    no;
-		register char  *lsave;
-
-		no = OP(scan) - OPEN;
-		lsave = reginput;
-
-		if (regmatch(nxt)) {
-		    /*
-		     * Don't set startp if some later invocation of the same
-		     * parentheses already has.
-		     */
-		    if (regstartp[no] == NULL)
-			regstartp[no] = lsave;
-		    return (1);
-		} else
+		break;
+	    case W_START:
+		if (*reginput == '\0' || isspace(*reginput))
 		    return (0);
-	    }
-	case CLOSE + 1:
-	case CLOSE + 2:
-	case CLOSE + 3:
-	case CLOSE + 4:
-	case CLOSE + 5:
-	case CLOSE + 6:
-	case CLOSE + 7:
-	case CLOSE + 8:
-	case CLOSE + 9:{
-		register int    no;
-		register char  *lsave;
-
-		no = OP(scan) - CLOSE;
-		lsave = reginput;
-
-		if (regmatch(nxt)) {
-		    /*
-		     * Don't set endp if some later invocation of the same
-		     * parentheses already has.
-		     */
-		    if (regendp[no] == NULL)
-			regendp[no] = lsave;
-		    return (1);
-		} else
+		if (reginput != regbol && !isspace(*(reginput - 1)))
 		    return (0);
-	    }
-	case BRANCH:{
-		register char  *lsave;
-
-		if (OP(nxt) != BRANCH)	/* No choice. */
-		    nxt = OPERAND(scan);	/* Avoid recursion. */
-		else {
-		    do {
-			lsave = reginput;
-			if (regmatch(OPERAND(scan)))
-			    return (1);
-			reginput = lsave;
-			scan = regnext(scan);
-		    } while (scan != NULL && OP(scan) == BRANCH);
+		break;
+	    case W_END:
+		if (*reginput != '\0' && !isspace(*reginput))
 		    return (0);
-		    /* NOTREACHED */
+		if (reginput == regbol || isspace(*(reginput - 1)))
+		    return (0);
+		break;
+	    case ANY:
+		if (*reginput == '\0')
+		    return (0);
+		reginput++;
+		break;
+	    case EXACTLY:{
+		    register int    len;
+		    register char  *opnd;
+
+		    opnd = OPERAND(scan);
+		    /* Inline the first character, for speed. */
+		    if (*opnd != *reginput)
+			return (0);
+		    len = strlen(opnd);
+		    if (len > 1 && strncmp(opnd, reginput, len) != 0)
+			return (0);
+		    reginput += len;
 		}
-	    }
-	    break;
-	case STAR:{
-		register char   nextch;
-		register int    no;
-		register char  *lsave;
-		register int    min;
+		break;
+	    case ANYOF:
+		if (*reginput == '\0' || strchr(OPERAND(scan), *reginput) == NULL)
+		    return (0);
+		reginput++;
+		break;
+	    case ANYBUT:
+		if (*reginput == '\0' || strchr(OPERAND(scan), *reginput) != NULL)
+		    return (0);
+		reginput++;
+		break;
+	    case NOTHING:
+		break;
+	    case BACK:
+		break;
+	    case OPEN + 1:
+	    case OPEN + 2:
+	    case OPEN + 3:
+	    case OPEN + 4:
+	    case OPEN + 5:
+	    case OPEN + 6:
+	    case OPEN + 7:
+	    case OPEN + 8:
+	    case OPEN + 9:{
+		    register int    no;
+		    register char  *save;
 
-		/*
-		 * Lookahead to avoid useless match attempts when we know
-		 * what character comes next.
-		 */
-		nextch = '\0';
-		if (OP(nxt) == EXACTLY)
-		    nextch = *OPERAND(nxt);
-		min = (OP(scan) == STAR) ? 0 : 1;
-		lsave = reginput;
-		no = regrepeat(OPERAND(scan));
-		while (no >= min) {
-		    /* If it could work, try it. */
-		    if (nextch == '\0' || *reginput == nextch)
-			if (regmatch(nxt))
-			    return (1);
-		    /* Couldn't or didn't -- back up. */
-		    no--;
-		    reginput = lsave + no;
+		    no = OP(scan) - OPEN;
+		    save = reginput;
+
+		    if (regmatch(next)) {
+
+			/*
+			 * Don't set startp if some later invocation of the
+			 * same parentheses already has.
+			 */
+			if (regstartp[no] == NULL)
+			    regstartp[no] = save;
+			return (1);
+		    } else
+			return (0);
 		}
+		break;
+	    case CLOSE + 1:
+	    case CLOSE + 2:
+	    case CLOSE + 3:
+	    case CLOSE + 4:
+	    case CLOSE + 5:
+	    case CLOSE + 6:
+	    case CLOSE + 7:
+	    case CLOSE + 8:
+	    case CLOSE + 9:{
+		    register int    no;
+		    register char  *save;
+
+		    no = OP(scan) - CLOSE;
+		    save = reginput;
+
+		    if (regmatch(next)) {
+
+			/*
+			 * Don't set endp if some later invocation of the
+			 * same parentheses already has.
+			 */
+			if (regendp[no] == NULL)
+			    regendp[no] = save;
+			return (1);
+		    } else
+			return (0);
+		}
+		break;
+	    case BRANCH:{
+		    register char  *save;
+
+		    if (OP(next) != BRANCH)	/* No choice. */
+			next = OPERAND(scan);	/* Avoid recursion. */
+		    else {
+			do {
+			    save = reginput;
+			    if (regmatch(OPERAND(scan)))
+				return (1);
+			    reginput = save;
+			    scan = regnext(scan);
+			} while (scan != NULL && OP(scan) == BRANCH);
+			return (0);
+			/* NOTREACHED */
+		    }
+		}
+		break;
+	    case STAR:
+	    case PLUS:{
+		    register char   nextch;
+		    register int    no;
+		    register char  *save;
+		    register int    min;
+
+		    /*
+		     * Lookahead to avoid useless match attempts when we know
+		     * what character comes next.
+		     */
+		    nextch = '\0';
+		    if (OP(next) == EXACTLY)
+			nextch = *OPERAND(next);
+		    min = (OP(scan) == STAR) ? 0 : 1;
+		    save = reginput;
+		    no = regrepeat(OPERAND(scan));
+		    while (no >= min) {
+			/* If it could work, try it. */
+			if (nextch == '\0' || *reginput == nextch)
+			    if (regmatch(next))
+				return (1);
+			/* Couldn't or didn't -- back up. */
+			no--;
+			reginput = save + no;
+		    }
+		    return (0);
+		}
+		break;
+	    case END:
+		return (1);	/* Success! */
+		break;
+	    default:
+		regerror("memory corruption");
 		return (0);
-	    }
-	case END:
-	    return (1);		/* Success! */
-	default:
-	    regerror("memory corruption");
-	    return (0);
+		break;
 	}
 
-	scan = nxt;
+	scan = next;
     }
 
     /*
@@ -1051,16 +1004,8 @@ char           *prog;
 /*
  - regrepeat - repeatedly match something simple, report how many
  */
-#ifdef __STDC__
-
-static int regrepeat(char *p)
-
-#else
-
-static int regrepeat(p)
-char           *p;
-
-#endif
+static int
+regrepeat(char *p)
 {
     register int    count = 0;
     register char  *scan;
@@ -1069,52 +1014,43 @@ char           *p;
     scan = reginput;
     opnd = OPERAND(p);
     switch (OP(p)) {
-    case ANY:
-	count = strlen(scan);
-	scan += count;
-	break;
-    case EXACTLY:
-	while (*opnd == *scan) {
-	    count++;
-	    scan++;
-	}
-	break;
-    case ANYOF:
-	while (*scan != '\0' && strchr(opnd, *scan) != NULL) {
-	    count++;
-	    scan++;
-	}
-	break;
-    case ANYBUT:
-	while (*scan != '\0' && strchr(opnd, *scan) == NULL) {
-	    count++;
-	    scan++;
-	}
-	break;
-    default:			/* Oh dear.  Called inappropriately. */
-	regerror("internal foulup");
-	count = 0;		/* Best compromise. */
-	break;
+	case ANY:
+	    count = strlen(scan);
+	    scan += count;
+	    break;
+	case EXACTLY:
+	    while (*opnd == *scan) {
+		count++;
+		scan++;
+	    }
+	    break;
+	case ANYOF:
+	    while (*scan != '\0' && strchr(opnd, *scan) != NULL) {
+		count++;
+		scan++;
+	    }
+	    break;
+	case ANYBUT:
+	    while (*scan != '\0' && strchr(opnd, *scan) == NULL) {
+		count++;
+		scan++;
+	    }
+	    break;
+	default:		/* Oh dear.  Called inappropriately. */
+	    regerror("internal foulup");
+	    count = 0;		/* Best compromise. */
+	    break;
     }
     reginput = scan;
 
     return (count);
 }
 
-
 /*
- - regnext - dig the "nxt" pointer out of a node
+ - regnext - dig the "next" pointer out of a node
  */
-#ifdef __STDC__
-
-static char *regnext(register char *p)
-
-#else
-
-static char *regnext(p)
-register char  *p;
-
-#endif
+static char    *
+regnext(register char *p)
 {
     register int    offset;
 
@@ -1132,37 +1068,28 @@ register char  *p;
 }
 
 #ifdef DEBUG
-
-STATIC char    *regprop();
+STATIC char    *regprop(char *);
 
 /*
  - regdump - dump a regexp onto stdout in vaguely comprehensible form
  */
-#ifdef __STDC__
-
-void regdump(regexp *r)
-
-#else
-
-void regdump(r)
-regexp         *r;
-
-#endif
+void
+regdump(regexp * r)
 {
     register char  *s;
     register char   op = EXACTLY;	/* Arbitrary non-END op. */
-    register char  *nxt;
+    register char  *next;
 
 
     s = r->program + 1;
     while (op != END) {		/* While that wasn't END last time... */
 	op = OP(s);
 	printf("%2d%s", s - r->program, regprop(s));	/* Where, what. */
-	nxt = regnext(s);
-	if (nxt == NULL)	/* nxt ptr. */
+	next = regnext(s);
+	if (next == NULL)	/* Next ptr. */
 	    printf("(0)");
 	else
-	    printf("(%d)", (s - r->program) + (nxt - s));
+	    printf("(%d)", (s - r->program) + (next - s));
 	s += 3;
 	if (op == ANYOF || op == ANYBUT || op == EXACTLY) {
 	    /* Literal string, where present. */
@@ -1188,96 +1115,90 @@ regexp         *r;
 /*
  - regprop - printable representation of opcode
  */
-#ifdef __STDC__
-
-static char *regprop(char *op)
-
-#else
-
-static char *regprop(op)
-char           *op;
-
-#endif
+static char    *
+regprop(char *op)
 {
     register char  *p;
     static char     buf[50];
 
-    strcpy(buf, ":");
+    (void) strcpy(buf, ":");
 
     switch (OP(op)) {
-    case BOL:
-	p = "BOL";
-	break;
-    case EOL:
-	p = "EOL";
-	break;
-#ifdef NN
-     case W_START:
-	p = "WSTART";
-	break;
-     case W_END:
-	p = "WEND";
-	break;
-#endif	
-    case ANY:
-	p = "ANY";
-	break;
-    case ANYOF:
-	p = "ANYOF";
-	break;
-    case ANYBUT:
-	p = "ANYBUT";
-	break;
-    case BRANCH:
-	p = "BRANCH";
-	break;
-    case EXACTLY:
-	p = "EXACTLY";
-	break;
-    case NOTHING:
-	p = "NOTHING";
-	break;
-    case BACK:
-	p = "BACK";
-	break;
-    case END:
-	p = "END";
-	break;
-    case OPEN + 1:
-    case OPEN + 2:
-    case OPEN + 3:
-    case OPEN + 4:
-    case OPEN + 5:
-    case OPEN + 6:
-    case OPEN + 7:
-    case OPEN + 8:
-    case OPEN + 9:
-	sprintf(buf + strlen(buf), "OPEN%d", OP(op) - OPEN);
-	p = NULL;
-	break;
-    case CLOSE + 1:
-    case CLOSE + 2:
-    case CLOSE + 3:
-    case CLOSE + 4:
-    case CLOSE + 5:
-    case CLOSE + 6:
-    case CLOSE + 7:
-    case CLOSE + 8:
-    case CLOSE + 9:
-	sprintf(buf + strlen(buf), "CLOSE%d", OP(op) - CLOSE);
-	p = NULL;
-	break;
-    case STAR:
-	p = "STAR";
-	break;
-    default:
-	regerror("corrupted opcode");
-	break;
+	case BOL:
+	    p = "BOL";
+	    break;
+	case EOL:
+	    p = "EOL";
+	    break;
+	case W_START:
+	    p = "WSTART";
+	    break;
+	case W_END:
+	    p = "WEND";
+	    break;
+	case ANY:
+	    p = "ANY";
+	    break;
+	case ANYOF:
+	    p = "ANYOF";
+	    break;
+	case ANYBUT:
+	    p = "ANYBUT";
+	    break;
+	case BRANCH:
+	    p = "BRANCH";
+	    break;
+	case EXACTLY:
+	    p = "EXACTLY";
+	    break;
+	case NOTHING:
+	    p = "NOTHING";
+	    break;
+	case BACK:
+	    p = "BACK";
+	    break;
+	case END:
+	    p = "END";
+	    break;
+	case OPEN + 1:
+	case OPEN + 2:
+	case OPEN + 3:
+	case OPEN + 4:
+	case OPEN + 5:
+	case OPEN + 6:
+	case OPEN + 7:
+	case OPEN + 8:
+	case OPEN + 9:
+	    sprintf(buf + strlen(buf), "OPEN%d", OP(op) - OPEN);
+	    p = NULL;
+	    break;
+	case CLOSE + 1:
+	case CLOSE + 2:
+	case CLOSE + 3:
+	case CLOSE + 4:
+	case CLOSE + 5:
+	case CLOSE + 6:
+	case CLOSE + 7:
+	case CLOSE + 8:
+	case CLOSE + 9:
+	    sprintf(buf + strlen(buf), "CLOSE%d", OP(op) - CLOSE);
+	    p = NULL;
+	    break;
+	case STAR:
+	    p = "STAR";
+	    break;
+	case PLUS:
+	    p = "PLUS";
+	    break;
+	default:
+	    regerror("corrupted opcode");
+	    break;
     }
     if (p != NULL)
-	strcat(buf, p);
+	(void) strcat(buf, p);
     return (buf);
 }
+
 #endif
 
 /*
@@ -1286,23 +1207,15 @@ char           *op;
  * about it; at least one public-domain implementation of those (highly
  * useful) string routines has been published on Usenet.
  */
+
 #ifdef STRCSPN
 /*
  * strcspn - find length of initial segment of s1 consisting entirely
  * of characters not from s2
  */
 
-#ifdef __STDC__
-
-static int strcspn(char *s1, char *s2)
-
-#else
-
-static int strcspn(s1, s2)
-char           *s1;
-char           *s2;
-
-#endif
+static int
+strcspn(char *s1, char *s2)
 {
     register char  *scan1;
     register char  *scan2;
@@ -1317,82 +1230,14 @@ char           *s2;
     }
     return (count);
 }
-#endif
-
-
-/*
- - regsub - perform substitutions after a regexp match
- */
-#ifdef __STDC__
-
-void regsub(regexp *prog, char *source, char *dest)
-
-#else
-
-void regsub(prog, source, dest)
-regexp         *prog;
-char           *source;
-char           *dest;
 
 #endif
+
+
+void
+regerror(char *s)
 {
-    register char  *src;
-    register char  *dst;
-    register char   c;
-    register int    no;
-    register int    len;
 
-    if (prog == NULL || source == NULL || dest == NULL) {
-	regerror("NULL parm to regsub");
-	return;
-    }
-    if (UCHARAT(prog->program) != MAGIC) {
-	regerror("damaged regexp fed to regsub");
-	return;
-    }
-    src = source;
-    dst = dest;
-    while ((c = *src++) != '\0') {
-	if (c == '&')
-	    no = 0;
-	else if (c == '\\' && '0' <= *src && *src <= '9')
-	    no = *src++ - '0';
-	else
-	    no = -1;
-
-	if (no < 0) {		/* Ordinary character. */
-	    if (c == '\\' && (*src == '\\' || *src == '&'))
-		c = *src++;
-	    *dst++ = c;
-	} else if (prog->startp[no] != NULL && prog->endp[no] != NULL) {
-	    len = prog->endp[no] - prog->startp[no];
-	    strncpy(dst, prog->startp[no], len);
-	    dst += len;
-	    if (len != 0 && *(dst - 1) == '\0') {	/* strncpy hit NUL. */
-		regerror("damaged match string");
-		return;
-	    }
-	}
-    }
-    *dst++ = '\0';
-}
-
-
-#ifdef __STDC__
-
-void regerror(char *s)
-
-#else
-
-void regerror(s)
-char           *s;
-
-#endif
-{
-#ifdef NN
-    msg("REGEXP ERROR: %s", s);
-#else
-    fprintf(stderr, "regexp(3): %s", s);
-    exit(1);
-#endif
+    msg("regexp: %s", s);
+    /* NOTREACHED */
 }
