@@ -1,19 +1,22 @@
 /*
  *	(c) Copyright 1990, Kim Fabricius Storm.  All rights reserved.
+ *      Copyright (c) 1996-2005 Michael T Pins.  All rights reserved.
  *
  *	Reply address rewriting.
  */
 
+#include <string.h>
+#include <ctype.h>
 #include "config.h"
+#include "global.h"
 
 
 int
-reroute(route, address)
-char *route, *address;
+reroute(char *route, char *address)
 {
-    char *name, *atpos;
-    register char *sp;
-    register c;
+    char           *name, *atpos;
+    register char  *sp;
+    register int    c;
 
     if ((atpos = strchr(address, '@'))) {
 	name = atpos;
@@ -23,10 +26,12 @@ char *route, *address;
 		name++;
 		break;
 	    }
-	if (name < address) name++;
+	if (name < address)
+	    name++;
 
 	for (sp = atpos; (c = *sp); sp++)
-	    if (isspace(c) || c == '>') break;
+	    if (isspace(c) || c == '>')
+		break;
 
 	*sp = NUL;
 	strcpy(route, name);
@@ -35,4 +40,3 @@ char *route, *address;
 	strcpy(route, address);
     return 1;
 }
-
